@@ -308,6 +308,7 @@ class EntityMetadata:
     unique_id_suffix: str | None = attrs.field(default=None)
     translation_key: str | None = attrs.field(default=None)
     fallback_name: str = attrs.field(validator=attrs.validators.instance_of(str))
+    primary: bool | None = attrs.field(default=None)
 
     def __attrs_post_init__(self) -> None:
         """Validate the entity metadata."""
@@ -347,6 +348,7 @@ class ZCLSensorMetadata(EntityMetadata):
     reporting_config: ReportingConfig | None = attrs.field(default=None)
     divisor: int | None = attrs.field(default=None)
     multiplier: int | None = attrs.field(default=None)
+    suggested_display_precision: int | None = attrs.field(default=None)
     unit: str | None = attrs.field(default=None)
     device_class: SensorDeviceClass | None = attrs.field(default=None)
     state_class: SensorStateClass | None = attrs.field(default=None)
@@ -444,6 +446,7 @@ class PreventDefaultEntityCreationMetadata:
 
     endpoint_id: int | None = attrs.field()
     cluster_id: int | None = attrs.field()
+    cluster_type: ClusterType | None = attrs.field()
     unique_id_suffix: str | None = attrs.field()
     function: Callable[Any, bool] | None = attrs.field()
 
@@ -560,6 +563,16 @@ class QuirkBuilder:
 
         UNBUILT_QUIRK_BUILDERS.append(self)
 
+    def _add_entity_metadata(self, entity_metadata: EntityMetadata) -> QuirkBuilder:
+        """Register new entity metadata and validate config."""
+        if entity_metadata.primary and any(
+            entity.primary for entity in self.entity_metadata
+        ):
+            raise ValueError("Only one primary entity can be defined per device")
+
+        self.entity_metadata.append(entity_metadata)
+        return self
+
     def applies_to(self, manufacturer: str, model: str) -> QuirkBuilder:
         """Register this quirks v2 entry for the specified manufacturer and model."""
         self.manufacturer_model_metadata.append(
@@ -772,12 +785,13 @@ class QuirkBuilder:
         unique_id_suffix: str | None = None,
         translation_key: str | None = None,
         fallback_name: str | None = None,
+        primary: bool | None = None,
     ) -> QuirkBuilder:
         """Add an EntityMetadata containing ZCLEnumMetadata and return self.
 
         This method allows exposing an enum based entity in Home Assistant.
         """
-        self.entity_metadata.append(
+        self._add_entity_metadata(
             ZCLEnumMetadata(
                 endpoint_id=endpoint_id,
                 cluster_id=cluster_id,
@@ -792,6 +806,7 @@ class QuirkBuilder:
                 fallback_name=fallback_name,
                 enum=enum_class,
                 attribute_name=attribute_name,
+                primary=primary,
             )
         )
         return self
@@ -804,6 +819,7 @@ class QuirkBuilder:
         endpoint_id: int = 1,
         divisor: int = 1,
         multiplier: int = 1,
+        suggested_display_precision: int = 1,
         entity_type: EntityType = EntityType.STANDARD,
         device_class: SensorDeviceClass | None = None,
         state_class: SensorStateClass | None = None,
@@ -815,12 +831,13 @@ class QuirkBuilder:
         unique_id_suffix: str | None = None,
         translation_key: str | None = None,
         fallback_name: str | None = None,
+        primary: bool | None = None,
     ) -> QuirkBuilder:
         """Add an EntityMetadata containing ZCLSensorMetadata and return self.
 
         This method allows exposing a sensor entity in Home Assistant.
         """
-        self.entity_metadata.append(
+        self._add_entity_metadata(
             ZCLSensorMetadata(
                 endpoint_id=endpoint_id,
                 cluster_id=cluster_id,
@@ -837,9 +854,11 @@ class QuirkBuilder:
                 attribute_converter=attribute_converter,
                 divisor=divisor,
                 multiplier=multiplier,
+                suggested_display_precision=suggested_display_precision,
                 unit=unit,
                 device_class=device_class,
                 state_class=state_class,
+                primary=primary,
             )
         )
         return self
@@ -862,12 +881,13 @@ class QuirkBuilder:
         unique_id_suffix: str | None = None,
         translation_key: str | None = None,
         fallback_name: str | None = None,
+        primary: bool | None = None,
     ) -> QuirkBuilder:
         """Add an EntityMetadata containing SwitchMetadata and return self.
 
         This method allows exposing a switch entity in Home Assistant.
         """
-        self.entity_metadata.append(
+        self._add_entity_metadata(
             SwitchMetadata(
                 endpoint_id=endpoint_id,
                 cluster_id=cluster_id,
@@ -885,6 +905,7 @@ class QuirkBuilder:
                 invert_attribute_name=invert_attribute_name,
                 off_value=off_value,
                 on_value=on_value,
+                primary=primary,
             )
         )
         return self
@@ -909,12 +930,13 @@ class QuirkBuilder:
         unique_id_suffix: str | None = None,
         translation_key: str | None = None,
         fallback_name: str | None = None,
+        primary: bool | None = None,
     ) -> QuirkBuilder:
         """Add an EntityMetadata containing NumberMetadata and return self.
 
         This method allows exposing a number entity in Home Assistant.
         """
-        self.entity_metadata.append(
+        self._add_entity_metadata(
             NumberMetadata(
                 endpoint_id=endpoint_id,
                 cluster_id=cluster_id,
@@ -935,6 +957,7 @@ class QuirkBuilder:
                 mode=mode,
                 multiplier=multiplier,
                 device_class=device_class,
+                primary=primary,
             )
         )
         return self
@@ -954,12 +977,13 @@ class QuirkBuilder:
         unique_id_suffix: str | None = None,
         translation_key: str | None = None,
         fallback_name: str | None = None,
+        primary: bool | None = None,
     ) -> QuirkBuilder:
         """Add an EntityMetadata containing BinarySensorMetadata and return self.
 
         This method allows exposing a binary sensor entity in Home Assistant.
         """
-        self.entity_metadata.append(
+        self._add_entity_metadata(
             BinarySensorMetadata(
                 endpoint_id=endpoint_id,
                 cluster_id=cluster_id,
@@ -975,6 +999,7 @@ class QuirkBuilder:
                 attribute_name=attribute_name,
                 attribute_converter=attribute_converter,
                 device_class=device_class,
+                primary=primary,
             )
         )
         return self
@@ -992,13 +1017,14 @@ class QuirkBuilder:
         unique_id_suffix: str | None = None,
         translation_key: str | None = None,
         fallback_name: str | None = None,
+        primary: bool | None = None,
     ) -> QuirkBuilder:
         """Add an EntityMetadata containing WriteAttributeButtonMetadata and return self.
 
         This method allows exposing a button entity in Home Assistant that writes
         a value to an attribute when pressed.
         """
-        self.entity_metadata.append(
+        self._add_entity_metadata(
             WriteAttributeButtonMetadata(
                 endpoint_id=endpoint_id,
                 cluster_id=cluster_id,
@@ -1012,6 +1038,7 @@ class QuirkBuilder:
                 fallback_name=fallback_name,
                 attribute_name=attribute_name,
                 attribute_value=attribute_value,
+                primary=primary,
             )
         )
         return self
@@ -1029,13 +1056,14 @@ class QuirkBuilder:
         unique_id_suffix: str | None = None,
         translation_key: str | None = None,
         fallback_name: str | None = None,
+        primary: bool | None = None,
     ) -> QuirkBuilder:
         """Add an EntityMetadata containing ZCLCommandButtonMetadata and return self.
 
         This method allows exposing a button entity in Home Assistant that executes
         a ZCL command when pressed.
         """
-        self.entity_metadata.append(
+        self._add_entity_metadata(
             ZCLCommandButtonMetadata(
                 endpoint_id=endpoint_id,
                 cluster_id=cluster_id,
@@ -1049,6 +1077,7 @@ class QuirkBuilder:
                 command_name=command_name,
                 args=command_args if command_args is not None else (),
                 kwargs=command_kwargs if command_kwargs is not None else frozendict(),
+                primary=primary,
             )
         )
         return self
@@ -1077,14 +1106,19 @@ class QuirkBuilder:
         *,
         endpoint_id: int | None = None,
         cluster_id: int | None = None,
+        cluster_type: ClusterType | None = None,
         unique_id_suffix: str | None = None,
         function: Callable[Any, bool] | None = None,
     ) -> QuirkBuilder:
         """Do not create default entities."""
+        if cluster_id is not None and cluster_type is None:
+            cluster_type = ClusterType.Server
+
         self.disabled_default_entities.append(
             PreventDefaultEntityCreationMetadata(
                 endpoint_id=endpoint_id,
                 cluster_id=cluster_id,
+                cluster_type=cluster_type,
                 unique_id_suffix=unique_id_suffix,
                 function=function,
             ),
