@@ -566,6 +566,7 @@ class TransmitOptions(enum.Flag):
 
     ACK = 1
     APS_Encryption = 2
+    FORCE_ROUTE_DISCOVERY = 3
 
 
 class PacketPriority(enum.IntEnum):
@@ -644,3 +645,43 @@ class ZigbeePacket(BaseDataclassMixin):
                 self.priority,
             )
         )
+
+
+@dataclasses.dataclass(frozen=True)
+class NetworkBeacon(BaseDataclassMixin):
+    pan_id: PanId
+    extended_pan_id: EUI64
+    channel: basic.uint8_t
+    permit_joining: bool
+    stack_profile: basic.uint8_t
+    nwk_update_id: basic.uint8_t
+    lqi: basic.uint8_t
+
+    # Migrate to kwarg-only once we drop 3.9
+    src: NWK | None = None
+    rssi: basic.int8s | None = None
+    depth: basic.uint8_t | None = None
+    router_capacity: bool | None = None
+    device_capacity: bool | None = None
+    protocol_version: basic.uint8_t | None = None
+
+
+@dataclasses.dataclass(frozen=True)
+class CapturedPacket(BaseDataclassMixin):
+    timestamp: datetime
+    rssi: float
+    lqi: basic.uint8_t
+    channel: basic.uint8_t
+    data: bytes
+
+    def compute_fcs(self) -> bytes:
+        crc = 0x0000
+
+        for c in self.data:
+            q = (crc ^ c) & 15  # Do low-order 4 bits
+            crc = (crc // 16) ^ (q * 0x1081)
+
+            q = (crc ^ (c // 16)) & 15  # And high 4 bits
+            crc = (crc // 16) ^ (q * 0x1081)
+
+        return crc.to_bytes(2, "little")
